package dev.inmo.postssystem.business_cases.post_creating.server

import com.benasher44.uuid.uuid4
import dev.inmo.postssystem.core.content.Content
import dev.inmo.postssystem.core.content.api.ContentRepo
import dev.inmo.postssystem.core.post.*
import dev.inmo.postssystem.core.post.repo.PostsRepo
import dev.inmo.postssystem.core.publishing.*

class BusinessPostCreatingCase(
    private val postsRepo: PostsRepo,
    private val contentRepo: ContentRepo,
    private val publishingRegistrar: PublishingRegistrar,
    private val postKeyGenerator: PostKeyGenerator = { _, _ -> uuid4().toString() },
    private val publishingKeyReceiverGetter: PublishingKeyReceiverGetter
) : PostCreatingCase {
    override suspend fun createPost(postContent: List<Content>, triggerId: TriggerId?): RegisteredPost? {
        val content = postContent.mapNotNull { contentRepo.registerContent(it) }
        val post = postsRepo.createPost(SimplePost(content.map { it.id })) ?: return null

        triggerId ?.let {
            val key = postKeyGenerator(post.id, triggerId)

            if (publishingRegistrar.registerTriggerForPost(key, post.id)) {
                publishingKeyReceiverGetter(it) ?.acceptKey(post.id, key)
            }
        }

        return post
    }
}
