package dev.inmo.postssystem.core.content

import dev.inmo.micro_utils.common.ByteArrayAllocator
import dev.inmo.micro_utils.common.ByteArrayAllocatorSerializer
import dev.inmo.micro_utils.mime_types.KnownMimeTypes
import dev.inmo.micro_utils.mime_types.MimeType
import kotlinx.serialization.Serializable

typealias ContentId = String

/**
 * Content which is planned to be registered in database
 */
@Serializable
sealed class Content

/**
 * It is content, which was added by some external source to use inside of some plugins. For example, you can use
 * location as content inside your target system and add this type of content specially for this system only
 */
@Serializable
data class SpecialContent(
    val internalId: ContentId
) : Content()

@Serializable
data class TextContent(
    val text: String
) : Content()

@Serializable
data class BinaryContent(
    val mimeType: MimeType,
    val originalFileName: String,
    @Serializable(ByteArrayAllocatorSerializer::class)
    val dataAllocator: ByteArrayAllocator
) : Content()

val BinaryContent.isImage: Boolean
    get() = mimeType is KnownMimeTypes.Image

/**
 * Content which is already registered in database. Using its [id] you can retrieve all known
 * [dev.inmo.postssystem.core.post.RegisteredPost]s by using
 * [dev.inmo.postssystem.core.post.repo.ReadPostsRepo.getPostsByContent]
 */
@Serializable
data class RegisteredContent(
    val id: ContentId,
    val content: Content
)
