package dev.inmo.postssystem.core.api

import dev.inmo.postssystem.core.content.*
import dev.inmo.postssystem.core.generateContentId
import kotlinx.serialization.json.Json
import kotlin.test.Test
import kotlin.test.assertEquals

class ContentSerialization {
    private val simpleTextTestEntries = 10
    private val simpleSpecialTestEntries = 10

    @Test
    fun test_that_content_correctly_serializing_and_deserializing() {
        val contents = (0 until simpleTextTestEntries).map {
            TextContent("Example$it")
        } + (0 until simpleSpecialTestEntries).map {
            SpecialContent("$it")
        }

        val registeredContentFakes = contents.map { content ->
            RegisteredContent(
                generateContentId(),
                content
            )
        }

        val stringified = registeredContentFakes.map {
            Json.encodeToString(RegisteredContent.serializer(), it)
        }

        val parsed = stringified.map {
            Json.decodeFromString(RegisteredContent.serializer(), it)
        }

        parsed.forEachIndexed { i, registeredContent -> assertEquals(registeredContentFakes[i], registeredContent) }
    }
}
