package dev.inmo.postssystem.core.ktor.client.content

import dev.inmo.postssystem.core.content.ContentId
import dev.inmo.postssystem.core.content.RegisteredContent
import dev.inmo.postssystem.core.content.api.ReadContentRepo
import dev.inmo.postssystem.core.ktor.*
import dev.inmo.micro_utils.ktor.client.uniget
import dev.inmo.micro_utils.ktor.common.includeQueryParams
import dev.inmo.micro_utils.ktor.common.standardKtorSerialFormat
import dev.inmo.micro_utils.pagination.*
import io.ktor.client.HttpClient
import io.ktor.client.request.get
import kotlinx.serialization.builtins.nullable

class ReadContentRepoKtorClient(
    private val baseUrl: String,
    private val client: HttpClient = HttpClient()
) : ReadContentRepo {
    override suspend fun getContentsIds(): Set<ContentId> = client.get<ByteArray>(
        "$baseUrl/$getContentsIdsRoute"
    ).let {
        standardKtorSerialFormat.decodeFromByteArray(contentIdsSerializer, it)
    }

    override suspend fun getContentById(id: ContentId): RegisteredContent? = client.uniget(
        "$baseUrl/$getContentByIdRoute/$id",
        RegisteredContent.serializer().nullable
    )

    override suspend fun getContentByPagination(pagination: Pagination): PaginationResult<RegisteredContent> = client.uniget(
        "$baseUrl/$getContentByPaginationRoute".includeQueryParams(pagination.asUrlQueryParts),
        registeredContentPaginationResultSerializer
    )
}