package dev.inmo.postssystem.core.ktor.client.content

import dev.inmo.postssystem.core.content.*
import dev.inmo.postssystem.core.content.api.WriteContentRepo
import dev.inmo.postssystem.core.ktor.*
import dev.inmo.micro_utils.ktor.client.*
import io.ktor.client.HttpClient
import kotlinx.coroutines.flow.Flow
import kotlinx.serialization.builtins.nullable
import kotlinx.serialization.builtins.serializer

class WriteContentRepoKtorClient(
    private val baseUrl: String,
    private val client: HttpClient = HttpClient()
) : WriteContentRepo {
    override val contentCreatedFlow: Flow<RegisteredContent> = client.createStandardWebsocketFlow(
        "$baseUrl/$contentCreatedFlowRoute",
        deserializer = RegisteredContent.serializer()
    )

    override val contentDeletedFlow: Flow<RegisteredContent> = client.createStandardWebsocketFlow(
        "$baseUrl/$contentDeletedFlowRoute",
        deserializer = RegisteredContent.serializer()
    )

    override suspend fun registerContent(content: Content): RegisteredContent? = client.unipost(
        "$baseUrl/$registerContentRoute",
        BodyPair(Content.serializer(), content),
        RegisteredContent.serializer().nullable
    )

    override suspend fun deleteContent(id: ContentId): Boolean = client.unipost(
        "$baseUrl/$deleteContentRoute",
        BodyPair(ContentId.serializer(), id),
        Boolean.serializer()
    )
}