package dev.inmo.postssystem.core.publishing

import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.post.RegisteredPost
import dev.inmo.postssystem.core.post.repo.PostsRepo
import dev.inmo.postssystem.core.publishing.repos.PublishedPostsWriteRepo
import dev.inmo.postssystem.core.publishing.repos.PublishingKeysRepo
import kotlinx.coroutines.channels.BroadcastChannel
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.asFlow

interface PublishingTrigger {
    val postingTriggeredFlow: Flow<RegisteredPost>

    suspend fun triggerPosting(
        triggerControlKey: TriggerControlKey
    ): PostId?
}

class BusinessPublishingTrigger(
    private val postsRepo: PostsRepo,
    private val publishedPostsRepo: PublishedPostsWriteRepo,
    private val publishingKeysRepo: PublishingKeysRepo
) : PublishingTrigger {
    private val postingTriggeredChannel: BroadcastChannel<RegisteredPost> = BroadcastChannel(Channel.BUFFERED)
    override val postingTriggeredFlow: Flow<RegisteredPost> = postingTriggeredChannel.asFlow()

    override suspend fun triggerPosting(triggerControlKey: TriggerControlKey): PostId? {
        val postId = publishingKeysRepo.getPostIdByTriggerControlKey(triggerControlKey) ?: return null
        publishingKeysRepo.unsetPostTriggerControlKey(postId)

        return postsRepo.getPostById(postId) ?.let { post ->
            publishedPostsRepo.registerPublishedPost(post) ?.let { publishedPost ->
                if (postsRepo.deletePost(postId)) {
                    postingTriggeredChannel.send(post)
                    postId
                } else {
                    publishedPostsRepo.unpublishPost(publishedPost.id)
                    null
                }
            }
        }
    }
}
