package dev.inmo.postssystem.core.publishing.repos

import dev.inmo.postssystem.core.post.PostId
import dev.inmo.postssystem.core.post.RegisteredPost
import dev.inmo.postssystem.core.post.repo.PostsRepo
import dev.inmo.postssystem.core.post.repo.ReadPostsRepo
import kotlinx.coroutines.flow.Flow

interface PublishedPostsReadRepo : ReadPostsRepo
interface PublishedPostsWriteRepo {
    val postPublishedFlow: Flow<RegisteredPost>
    val postUnpublishedFlow: Flow<RegisteredPost>

    suspend fun registerPublishedPost(registeredPost: RegisteredPost): RegisteredPost?
    suspend fun unpublishPost(postId: PostId): Boolean?
}

interface PublishedPostsRepo : PublishedPostsReadRepo, PublishedPostsWriteRepo

class BusinessPublishedPostsRepo(
    private val realPostsRepoWithPublishedPosts: PostsRepo
) : PublishedPostsRepo, ReadPostsRepo by realPostsRepoWithPublishedPosts {
    override val postPublishedFlow: Flow<RegisteredPost>
        get() = realPostsRepoWithPublishedPosts.postCreatedFlow
    override val postUnpublishedFlow: Flow<RegisteredPost>
        get() = realPostsRepoWithPublishedPosts.postDeletedFlow

    override suspend fun registerPublishedPost(
        registeredPost: RegisteredPost
    ): RegisteredPost? = realPostsRepoWithPublishedPosts.createPost(registeredPost)

    override suspend fun unpublishPost(
        postId: PostId
    ): Boolean? = realPostsRepoWithPublishedPosts.deletePost(postId)

}
